@@ -1,17 +1,20 @@
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
+from copy import copy
+
 import matplotlib.colors as colors
 import matplotlib.pyplot as plt
 import numpy as np
-from copy import copy
 import plotly.graph_objects as go
-from geoh5py.objects import Curve, Grid2D, Points, Surface, BlockModel
-from geoh5py.data import Data
+from geoh5py.data import Data, ReferencedData
+from geoh5py.objects import BlockModel, Curve, Grid2D, Points, Surface
 
-from geoapps.utils import (
-    filter_xy,
-    format_labels,
-    symlog,
-    inv_symlog,
-)
+from geoapps.utils.utils import filter_xy, format_labels, inv_symlog, symlog
 
 
 def normalize(values):
@@ -80,7 +83,11 @@ def plot_plan_data_selection(entity, data, **kwargs):
     # for collection in axis.collections:
     #     collection.remove()
 
-    locations = entity.vertices
+    if getattr(entity, "vertices", None) is not None:
+        locations = entity.vertices
+    else:
+        locations = entity.centroids
+
     if "resolution" not in kwargs.keys():
         resolution = 0
     else:
@@ -95,8 +102,12 @@ def plot_plan_data_selection(entity, data, **kwargs):
         data.values[0], str
     ):
         values = np.asarray(data.values, dtype=float).copy()
-        values[(values > 1e-18) * (values < 2e-18)] = np.nan
         values[values == -99999] = np.nan
+    elif isinstance(data, str) and (data in "XYZ"):
+        values = locations[:, "XYZ".index(data)]
+
+    if values is not None and (values.shape[0] != locations.shape[0]):
+        values = None
 
     color_norm = None
     if "color_norm" in kwargs.keys():
@@ -115,7 +126,9 @@ def plot_plan_data_selection(entity, data, **kwargs):
         map_vals = new_cmap["Value"].copy()
         cmap = colors.ListedColormap(
             np.c_[
-                new_cmap["Red"] / 255, new_cmap["Green"] / 255, new_cmap["Blue"] / 255,
+                new_cmap["Red"] / 255,
+                new_cmap["Green"] / 255,
+                new_cmap["Blue"] / 255,
             ]
         )
         color_norm = colors.BoundaryNorm(map_vals, cmap.N)
@@ -157,12 +170,14 @@ def plot_plan_data_selection(entity, data, **kwargs):
     else:
         x, y = entity.vertices[:, 0], entity.vertices[:, 1]
         if indices is None:
-            indices = filter_xy(x, y, resolution, window=window,)
+            indices = filter_xy(
+                x,
+                y,
+                resolution,
+                window=window,
+            )
         X, Y = x[indices], y[indices]
 
-        if data == "Z":
-            values = entity.vertices[:, 2]
-
         if values is not None:
             values = values[indices]
 
@@ -196,19 +211,16 @@ def plot_plan_data_selection(entity, data, **kwargs):
         ind = ~np.isnan(values.ravel())
         x = X.ravel()[ind]
         y = Y.ravel()[ind]
-        if ind.sum() > 0:
-            format_labels(x, y, axis, **kwargs)
-            axis.set_xlim([x.min(), x.max()])
-            axis.set_ylim([y.min(), y.max()])
-    elif np.any(x) and np.any(y):
-        format_labels(x, y, axis, **kwargs)
-        axis.set_xlim([x.min(), x.max()])
-        axis.set_ylim([y.min(), y.max()])
 
-    if (
-        "colorbar" in kwargs.keys()
-        and values[~np.isnan(values)].min() != values[~np.isnan(values)].max()
-    ):
+    if np.any(x) and np.any(y):
+        width = x.max() - x.min()
+        height = y.max() - y.min()
+
+        format_labels(x, y, axis, **kwargs)
+        axis.set_xlim([x.min() - width * 0.1, x.max() + width * 0.1])
+        axis.set_ylim([y.min() - height * 0.1, y.max() + height * 0.1])
+
+    if "colorbar" in kwargs.keys() and kwargs["colorbar"]:
         plt.colorbar(out, ax=axis)
 
     line_selection = np.zeros_like(indices, dtype=bool)
@@ -217,15 +229,23 @@ def plot_plan_data_selection(entity, data, **kwargs):
     ):
         for key, values in kwargs["highlight_selection"].items():
 
-            if not np.any(entity.get_data(key)):
+            if not np.any(entity.workspace.get_entity(key)):
                 continue
 
+            line_data = entity.workspace.get_entity(key)[0]
+            if isinstance(line_data, ReferencedData):
+                values = [
+                    key
+                    for key, value in line_data.value_map.map.items()
+                    if value in values
+                ]
+
             for line in values:
-                ind = np.where(entity.get_data(key)[0].values == line)[0]
+                ind = np.where(line_data.values == line)[0]
                 x, y, values = (
                     locations[ind, 0],
                     locations[ind, 1],
-                    entity.get_data(key)[0].values[ind],
+                    entity.workspace.get_entity(key)[0].values[ind],
                 )
                 ind_line = filter_xy(x, y, resolution, window=window)
                 axis.scatter(x[ind_line], y[ind_line], marker_size * 2, "k", marker="+")
@@ -258,15 +278,19 @@ def plot_profile_data_selection(
 
         for line in values:
 
-            if entity.get_data(key):
-                ind = np.where(entity.get_data(key)[0].values == line)[0]
+            if entity.workspace.get_entity(key):
+                ind = np.where(entity.workspace.get_entity(key)[0].values == line)[0]
             else:
                 continue
             if len(ind) == 0:
                 continue
 
             if resolution is not None:
-                dwn_ind = filter_xy(locations[ind, 0], locations[ind, 1], resolution,)
+                dwn_ind = filter_xy(
+                    locations[ind, 0],
+                    locations[ind, 1],
+                    resolution,
+                )
 
                 ind = ind[dwn_ind]
 
@@ -285,10 +309,10 @@ def plot_profile_data_selection(
 
             for ii, field in enumerate(field_list):
                 if (
-                    entity.get_data(field)
-                    and entity.get_data(field)[0].values is not None
+                    entity.workspace.get_entity(field)
+                    and entity.workspace.get_entity(field)[0].values is not None
                 ):
-                    values = entity.get_data(field)[0].values[ind][order]
+                    values = entity.workspace.get_entity(field)[0].values[ind][order]
 
                     xx.append(dist[order][~np.isnan(values)])
                     yy.append(values[~np.isnan(values)])
@@ -374,7 +398,7 @@ def plotly_scatter(
 
 
 def plotly_surface(
-    surface, figure=None, intensity=None, colorscale="Portland", **kwargs
+    surface, figure=None, intensity=None, colorscale="Portland", colorbar=True, **kwargs
 ):
     """
     Create a plotly.graph_objects.Mesh3D figure.
@@ -402,6 +426,7 @@ def plotly_surface(
         figure.data[-1].intensity = intensity
 
     figure.update_layout(scene_aspectmode="data")
+    figure.update_traces(showscale=colorbar)
 
     return figure
 
@@ -481,7 +506,7 @@ def check_data_type(data):
 #
 #     curves = [
 #         entity.parent.name + "." + entity.name
-#         for entity in workspace.all_objects()
+#         for entity in workspace.objects
 #         if isinstance(entity, Curve)
 #     ]
 #     names = [name for name in sorted(curves)]
