@@ -1,22 +1,31 @@
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
 import os
+
 import numpy as np
-import plotly.graph_objects as go
 import plotly.express as px
+import plotly.graph_objects as go
 from ipywidgets import (
-    Dropdown,
     Checkbox,
-    IntSlider,
+    Dropdown,
     FloatText,
-    VBox,
     HBox,
-    ToggleButton,
-    interactive_output,
-    Layout,
+    IntSlider,
     Label,
+    Layout,
+    ToggleButton,
+    VBox,
+    interactive_output,
 )
-from geoapps.utils import symlog, random_sampling
+
+from geoapps.plotting import format_axis, normalize
 from geoapps.selection import ObjectDataSelection
-from geoapps.plotting import normalize, format_axis
+from geoapps.utils.utils import random_sampling, symlog
 
 
 class ScatterPlots(ObjectDataSelection):
@@ -26,49 +35,67 @@ class ScatterPlots(ObjectDataSelection):
 
     defaults = {
         "h5file": "../../assets/FlinFlon.geoh5",
-        "objects": "geochem",
-        "data": ["Al2O3", "CaO", "V", "MgO", "Ba"],
-        "x": "Al2O3",
-        "y": "CaO",
-        "z": "Ba",
+        "objects": "{79b719bc-d996-4f52-9af0-10aa9c7bb941}",
+        "data": [
+            "{18c2560c-6161-468a-8571-5d9d59649535}",
+            "{41d51965-3670-43ba-8a10-d399070689e3}",
+            "{94a150e8-16d9-4784-a7aa-e6271df3a3ef}",
+            "{cb35da1c-7ea4-44f0-8817-e3d80e8ba98c}",
+            "{cdd7668a-4b5b-49ac-9365-c9ce4fddf733}",
+        ],
+        "x": "{cdd7668a-4b5b-49ac-9365-c9ce4fddf733}",
+        "x_active": True,
+        "y": "{18c2560c-6161-468a-8571-5d9d59649535}",
+        "y_active": True,
+        "z": "{cb35da1c-7ea4-44f0-8817-e3d80e8ba98c}",
         "y_log": True,
         "z_log": True,
         "z_active": True,
-        "color": "V",
+        "color": "{94a150e8-16d9-4784-a7aa-e6271df3a3ef}",
         "color_active": True,
         "color_log": True,
-        "size": "MgO",
+        "size": "{41d51965-3670-43ba-8a10-d399070689e3}",
         "size_active": True,
         "color_maps": "inferno",
         "refresh": True,
-        "refresh_trigger": True,
+        "refresh": True,
     }
 
-    def __init__(self, static=False, **kwargs):
-        self.static = static
-        self.select_multiple = True
-        self._add_groups = True
+    _select_multiple = True
+    _add_groups = False
+    _downsampling = None
+    _color = None
+    _x = None
+    _y = None
+    _z = None
+    _size = None
+
+    def __init__(self, **kwargs):
+        self.defaults = self.update_defaults(**kwargs)
         self.custom_colormap = []
         self._indices = None
 
         def channel_bounds_setter(caller):
             self.set_channel_bounds(caller["owner"].name)
 
-        self._downsampling = IntSlider(
-            description="Population Downsampling:",
-            min=1,
-            style={"description_width": "initial"},
-            continuous_update=False,
-        )
-
-        self._x = Dropdown(description="Data:")
-        self._x.observe(channel_bounds_setter, names="value")
-        self._x.name = "x"
+        self.x.observe(channel_bounds_setter, names="value")
+        self.x.name = "x"
         self._x_active = Checkbox(description="Active", value=True, indent=False)
-        self._x_log = Checkbox(description="Log10", value=False, indent=False,)
-        self._x_thresh = FloatText(description="Threshold", value=1e-1, indent=False,)
-        self._x_min = FloatText(description="Min", indent=False,)
-        self._x_max = FloatText(description="Max", indent=False,)
+        self._x_log = Checkbox(
+            description="Log10",
+            value=False,
+            indent=False,
+        )
+        self._x_thresh = FloatText(
+            description="Threshold",
+            value=1e-1,
+        )
+        self._x_min = FloatText(
+            description="Min",
+        )
+        self._x_max = FloatText(
+            description="Max",
+        )
         self._x_panel = VBox(
             [
                 self._x_active,
@@ -77,15 +104,24 @@ class ScatterPlots(ObjectDataSelection):
                 HBox([self._x_min, self._x_max]),
             ]
         )
-
-        self._y = Dropdown(description="Data:")
-        self._y.observe(channel_bounds_setter, names="value")
-        self._y.name = "y"
-        self._y_active = Checkbox(description="Active", value=True, indent=False,)
+        self.y.observe(channel_bounds_setter, names="value")
+        self.y.name = "y"
+        self._y_active = Checkbox(
+            description="Active",
+            value=True,
+            indent=False,
+        )
         self._y_log = Checkbox(description="Log10", value=False, indent=False)
-        self._y_thresh = FloatText(description="Threshold", value=1e-1, indent=False,)
-        self._y_min = FloatText(description="Min", indent=False,)
-        self._y_max = FloatText(description="Max", indent=False,)
+        self._y_thresh = FloatText(
+            description="Threshold",
+            value=1e-1,
+        )
+        self._y_min = FloatText(
+            description="Min",
+        )
+        self._y_max = FloatText(
+            description="Max",
+        )
         self._y_panel = VBox(
             [
                 self._y_active,
@@ -94,15 +130,28 @@ class ScatterPlots(ObjectDataSelection):
                 HBox([self._y_min, self._y_max]),
             ]
         )
-
-        self._z = Dropdown(description="Data:")
-        self._z.observe(channel_bounds_setter, names="value")
-        self._z.name = "z"
-        self._z_active = Checkbox(description="Active", value=False, indent=False,)
-        self._z_log = Checkbox(description="Log10", value=False, indent=False,)
-        self._z_thresh = FloatText(description="Threshold", value=1e-1, indent=False,)
-        self._z_min = FloatText(description="Min", indent=False,)
-        self._z_max = FloatText(description="Max", indent=False,)
+        self.z.observe(channel_bounds_setter, names="value")
+        self.z.name = "z"
+        self._z_active = Checkbox(
+            description="Active",
+            value=False,
+            indent=False,
+        )
+        self._z_log = Checkbox(
+            description="Log10",
+            value=False,
+            indent=False,
+        )
+        self._z_thresh = FloatText(
+            description="Threshold",
+            value=1e-1,
+        )
+        self._z_min = FloatText(
+            description="Min",
+        )
+        self._z_max = FloatText(
+            description="Max",
+        )
         self._z_panel = VBox(
             [
                 self._z_active,
@@ -111,22 +160,33 @@ class ScatterPlots(ObjectDataSelection):
                 HBox([self._z_min, self._z_max]),
             ]
         )
-
-        self._color = Dropdown(description="Data:")
-        self._color.observe(channel_bounds_setter, names="value")
-        self._color.name = "color"
-        self._color_log = Checkbox(description="Log10", value=False, indent=False,)
+        self.color.observe(channel_bounds_setter, names="value")
+        self.color.name = "color"
+        self._color_log = Checkbox(
+            description="Log10",
+            value=False,
+            indent=False,
+        )
         self._color_thresh = FloatText(
-            description="Threshold", value=1e-1, indent=False,
+            description="Threshold",
+            value=1e-1,
+        )
+        self._color_active = Checkbox(
+            description="Active",
+            value=False,
+            indent=False,
         )
-        self._color_active = Checkbox(description="Active", value=False, indent=False,)
         self._color_maps = Dropdown(
             description="Colormaps",
             options=px.colors.named_colorscales(),
             value="viridis",
         )
-        self._color_min = FloatText(description="Min", indent=False,)
-        self._color_max = FloatText(description="Max", indent=False,)
+        self._color_min = FloatText(
+            description="Min",
+        )
+        self._color_max = FloatText(
+            description="Max",
+        )
         self._color_panel = VBox(
             [
                 self._color_active,
@@ -136,20 +196,32 @@ class ScatterPlots(ObjectDataSelection):
                 HBox([self._color_min, self._color_max]),
             ]
         )
-
-        self._size = Dropdown(description="Data:")
-        self._size.observe(channel_bounds_setter, names="value")
-        self._size.name = "size"
-        self._size_active = Checkbox(description="Active", value=False, indent=False,)
-        self._size_log = Checkbox(description="Log10", value=False, indent=False,)
+        self.size.observe(channel_bounds_setter, names="value")
+        self.size.name = "size"
+        self._size_active = Checkbox(
+            description="Active",
+            value=False,
+            indent=False,
+        )
+        self._size_log = Checkbox(
+            description="Log10",
+            value=False,
+            indent=False,
+        )
         self._size_thresh = FloatText(
-            description="Threshold", value=1e-1, indent=False,
+            description="Threshold",
+            value=1e-1,
         )
         self._size_markers = IntSlider(
             min=1, max=100, value=20, description="Marker size", continuous_update=False
         )
-        self._size_min = FloatText(description="Min", indent=False,)
-        self._size_max = FloatText(description="Max", indent=False,)
+        self._size_min = FloatText(
+            description="Min",
+        )
+        self._size_max = FloatText(
+            description="Max",
+        )
+
         self._size_panel = VBox(
             [
                 self._size_active,
@@ -159,7 +231,7 @@ class ScatterPlots(ObjectDataSelection):
                 HBox([self._size_min, self._size_max]),
             ]
         )
-        self._refresh_trigger = ToggleButton(description="Refresh Plot", value=False)
+        self._refresh = ToggleButton(description="Refresh Plot", value=False)
 
         # Wrap all axis panels into dropdown
         def axes_pannels_trigger(_):
@@ -182,11 +254,7 @@ class ScatterPlots(ObjectDataSelection):
         self.data.observe(self.update_choices, names="value")
         self.objects.observe(self.update_objects, names="value")
         self.downsampling.observe(self.update_downsampling, names="value")
-
-        super().__init__(**self.apply_defaults(**kwargs))
-
         self.figure = go.FigureWidget()
-
         self.crossplot = interactive_output(
             self.plot_selection,
             {
@@ -222,37 +290,13 @@ class ScatterPlots(ObjectDataSelection):
                 "size_markers": self.size_markers,
                 "size_min": self.size_min,
                 "size_max": self.size_max,
-                "refresh_trigger": self.refresh_trigger,
+                "refresh": self.refresh,
             },
         )
-
-        def write_html(_):
-            self.write_html()
-
-        self.trigger.on_click(write_html)
+        self.trigger.on_click(self.trigger_click)
         self.trigger.description = "Save HTML"
 
-        if self.static:
-            self._main = VBox(
-                [
-                    self.project_panel,
-                    HBox([self.objects, self.data]),
-                    self.downsampling,
-                    self.axes_options,
-                    self.trigger,
-                ]
-            )
-        else:
-            self._main = VBox(
-                [
-                    self.project_panel,
-                    HBox([self.objects, self.data]),
-                    VBox([Label("Downsampling"), self.downsampling]),
-                    self.axes_options,
-                    self.trigger,
-                    self.figure,
-                ]
-            )
+        super().__init__(**self.defaults)
 
     @property
     def n_values(self):
@@ -286,8 +330,11 @@ class ScatterPlots(ObjectDataSelection):
     @property
     def color(self):
         """
-        :obj:`ipywidgets.Dropdown`
+        Widget for the selection of color values
         """
+        if getattr(self, "_color", None) is None:
+            self._color = Dropdown(description="Data:")
+
         return self._color
 
     @property
@@ -333,24 +380,48 @@ class ScatterPlots(ObjectDataSelection):
         return self._color_max
 
     @property
-    def downsampling(self):
+    def downsampling(self) -> IntSlider:
         """
-        :obj:`ipywidgets.IntSlider`
+        Widget controlling the size of the population.
         """
+        if getattr(self, "_downsampling", None) is None:
+            self._downsampling = IntSlider(
+                description="Population Downsampling:",
+                min=1,
+                style={"description_width": "initial"},
+                continuous_update=False,
+            )
         return self._downsampling
 
     @property
-    def refresh_trigger(self):
-        """
+    def main(self):
+        if self._main is None:
+            self._main = VBox(
+                [
+                    self.project_panel,
+                    HBox([self.objects, self.data]),
+                    VBox([Label("Downsampling"), self.downsampling]),
+                    self.axes_options,
+                    self.refresh,
+                    self.figure,
+                    self.trigger,
+                ]
+            )
 
-        """
-        return self._refresh_trigger
+        return self._main
+
+    @property
+    def refresh(self):
+        """"""
+        return self._refresh
 
     @property
     def size(self):
         """
-        :obj:`ipywidgets.Dropdown`
+        Widget for the selection of size scaling values
         """
+        if getattr(self, "_size", None) is None:
+            self._size = Dropdown(description="Data:")
         return self._size
 
     @property
@@ -398,8 +469,10 @@ class ScatterPlots(ObjectDataSelection):
     @property
     def x(self):
         """
-        :obj:`ipywidgets.Dropdown`
+        Widget for the selection of x-axis values
         """
+        if getattr(self, "_x", None) is None:
+            self._x = Dropdown(description="Data:")
         return self._x
 
     @property
@@ -440,8 +513,10 @@ class ScatterPlots(ObjectDataSelection):
     @property
     def y(self):
         """
-        :obj:`ipywidgets.Dropdown`
+        Widget for the selection of y-axis values
         """
+        if getattr(self, "_y", None) is None:
+            self._y = Dropdown(description="Data:")
         return self._y
 
     @property
@@ -482,8 +557,10 @@ class ScatterPlots(ObjectDataSelection):
     @property
     def z(self):
         """
-        :obj:`ipywidgets.Dropdown`
+        Widget for the selection of z-axis values
         """
+        if getattr(self, "_z", None) is None:
+            self._z = Dropdown(description="Data:")
         return self._z
 
     @property
@@ -535,15 +612,16 @@ class ScatterPlots(ObjectDataSelection):
 
         if channel not in self.data_channels.keys():
 
-            if obj.get_data(channel):
-                values = np.asarray(obj.get_data(channel)[0].values, dtype=float).copy()
-                values[(values > 1e-38) * (values < 2e-38)] = np.nan
-            elif channel == "Z":
+            if self.workspace.get_entity(channel):
+                values = np.asarray(
+                    self.workspace.get_entity(channel)[0].values, dtype=float
+                ).copy()
+            elif channel in "XYZ":
                 # Check number of points
                 if hasattr(obj, "centroids"):
-                    values = obj.centroids[:, 2]
+                    values = obj.centroids[:, "XYZ".index(channel)]
                 elif hasattr(obj, "vertices"):
-                    values = obj.vertices[:, 2]
+                    values = obj.vertices[:, "XYZ".index(channel)]
             else:
                 return
 
@@ -603,14 +681,10 @@ class ScatterPlots(ObjectDataSelection):
         size_markers,
         size_min,
         size_max,
-        refresh_trigger,
+        refresh,
     ):
 
-        if (
-            not self.refresh_trigger.value
-            or not self.refresh.value
-            or self.indices is None
-        ):
+        if not self.refresh.value or self.indices is None:
             return None
 
         if (
@@ -674,33 +748,33 @@ class ScatterPlots(ObjectDataSelection):
                 inbound = (x_axis >= x_min) * (x_axis <= x_max)
                 x_axis[~inbound] = np.nan
                 x_axis, x_label, x_ticks, x_ticklabels = format_axis(
-                    x, x_axis, x_log, x_thresh
+                    self.data.uid_name_map[x], x_axis, x_log, x_thresh
                 )
             else:
                 inbound = (z_axis >= z_min) * (z_axis <= z_max)
                 z_axis[~inbound] = np.nan
                 x_axis, x_label, x_ticks, x_ticklabels = format_axis(
-                    z, z_axis, z_log, z_thresh
+                    self.data.uid_name_map[z], z_axis, z_log, z_thresh
                 )
 
             if y_axis is not None:
                 inbound = (y_axis >= y_min) * (y_axis <= y_max)
                 y_axis[~inbound] = np.nan
                 y_axis, y_label, y_ticks, y_ticklabels = format_axis(
-                    y, y_axis, y_log, y_thresh
+                    self.data.uid_name_map[y], y_axis, y_log, y_thresh
                 )
             else:
                 inbound = (z_axis >= z_min) * (z_axis <= z_max)
                 z_axis[~inbound] = np.nan
                 y_axis, y_label, y_ticks, y_ticklabels = format_axis(
-                    z, z_axis, z_log, z_thresh
+                    self.data.uid_name_map[z], z_axis, z_log, z_thresh
                 )
 
             if z_axis is not None:
                 inbound = (z_axis >= z_min) * (z_axis <= z_max)
                 z_axis[~inbound] = np.nan
                 z_axis, z_label, z_ticks, z_ticklabels = format_axis(
-                    z, z_axis, z_log, z_thresh
+                    self.data.uid_name_map[z], z_axis, z_log, z_thresh
                 )
 
             if self.custom_colormap:
@@ -770,7 +844,7 @@ class ScatterPlots(ObjectDataSelection):
             self.figure.data = []
 
     def update_axes(self, refresh_plot=True):
-        self.refresh_trigger.value = False
+
         for name in [
             "x",
             "y",
@@ -778,19 +852,22 @@ class ScatterPlots(ObjectDataSelection):
             "color",
             "size",
         ]:
+            self.refresh.value = False
             widget = getattr(self, "_" + name)
             val = widget.value
-            widget.options = list(self.data_channels.keys())
+            widget.options = [
+                [self.data.uid_name_map[key], key] for key in self.data_channels
+            ]
 
-            if val in widget.options:
+            if val in list(dict(widget.options).values()):
                 widget.value = val
             else:
                 widget.value = None
         if refresh_plot:
-            self.refresh_trigger.value = True
+            self.refresh.value = True
 
     def update_choices(self, _):
-        self.refresh_trigger.value = False
+        self.refresh.value = False
 
         for channel in self.data.value:
             self.get_channel(channel)
@@ -805,14 +882,14 @@ class ScatterPlots(ObjectDataSelection):
         if self.downsampling.value != self.n_values:
             self.update_downsampling(None, refresh_plot=False)
 
-        self.refresh_trigger.value = True
+        self.refresh.value = True
 
     def update_downsampling(self, _, refresh_plot=True):
 
         if not list(self.data_channels.values()):
             return
 
-        self.refresh_trigger.value = False
+        self.refresh.value = False
         values = []
         for axis in [self.x, self.y, self.z]:
             vals = self.get_channel(axis.value)
@@ -831,18 +908,31 @@ class ScatterPlots(ObjectDataSelection):
         )[:, None]
         values[nans] = np.nan
         self._indices = random_sampling(
-            values.T, self.downsampling.value, bandwidth=2.0, rtol=1e0, method="hist",
+            values.T,
+            self.downsampling.value,
+            bandwidth=2.0,
+            rtol=1e0,
+            method="histogram",
         )
-        self.refresh_trigger.value = refresh_plot
+        self.refresh.value = refresh_plot
 
     def update_objects(self, _):
         self.data_channels = {}
-        self.downsampling.max = self.n_values
-        self.downsampling.value = np.min([5000, self.n_values])
+        self.refresh.value = False
+        self.figure.data = []
+        self.x_active.value = False
+        self.y_active.value = False
+        self.z_active.value = False
+        self.color_active.value = False
+        self.size_active.value = False
+        if self.n_values is not None:
+            self.downsampling.max = self.n_values
+            self.downsampling.value = np.min([5000, self.n_values])
         self._indices = None
         self.update_downsampling(None, refresh_plot=False)
+        self.refresh.value = True
 
-    def write_html(self):
+    def trigger_click(self, _):
         self.figure.write_html(
             os.path.join(
                 os.path.abspath(os.path.dirname(self.h5file)), "Crossplot.html"
