@@ -1,7 +1,12 @@
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
 from .calculator import Calculator
-from .create import Surface2D, ContourValues
+from .clustering import Clustering
 from .coordinate_transformation import CoordinateTransformation
 from .data_interpolation import DataInterpolation
 from .edge_detection import EdgeDetectionApp
-from .peak_finder import PeakFinder
-from .clustering import Clustering
