@@ -1,25 +1,34 @@
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+from uuid import UUID
+
+import numpy as np
+import pandas as pd
+import plotly.graph_objects as go
+from IPython.display import display
 from ipywidgets import (
     Button,
     Checkbox,
-    VBox,
-    HBox,
-    interactive_output,
     ColorPicker,
     Dropdown,
-    ToggleButtons,
-    Layout,
-    IntSlider,
     FloatText,
+    HBox,
+    IntSlider,
     Label,
+    Layout,
+    ToggleButtons,
+    VBox,
+    interactive_output,
 )
-from IPython.display import display
-import plotly.graph_objects as go
-import numpy as np
-from geoapps.plotting import ScatterPlots
-from geoapps.utils import random_sampling, hex_to_rgb, colors
-from sklearn.cluster import KMeans
 from scipy.spatial import cKDTree
-import pandas as pd
+from sklearn.cluster import KMeans
+
+from geoapps.plotting import ScatterPlots
+from geoapps.utils.utils import colors, hex_to_rgb, random_sampling
 
 
 class Clustering(ScatterPlots):
@@ -29,22 +38,28 @@ class Clustering(ScatterPlots):
 
     defaults = {
         "h5file": r"../../assets/FlinFlon.geoh5",
-        "objects": "geochem",
-        "data": ["Al2O3", "CaO", "V", "MgO", "Ba"],
-        "x": "Al2O3",
-        "y": "CaO",
-        "z": "Ba",
+        "objects": "{79b719bc-d996-4f52-9af0-10aa9c7bb941}",
+        "data": [
+            "{18c2560c-6161-468a-8571-5d9d59649535}",
+            "{41d51965-3670-43ba-8a10-d399070689e3}",
+            "{94a150e8-16d9-4784-a7aa-e6271df3a3ef}",
+            "{cb35da1c-7ea4-44f0-8817-e3d80e8ba98c}",
+            "{cdd7668a-4b5b-49ac-9365-c9ce4fddf733}",
+        ],
+        "x": "{cdd7668a-4b5b-49ac-9365-c9ce4fddf733}",
+        "x_active": True,
+        "y": "{18c2560c-6161-468a-8571-5d9d59649535}",
+        "y_active": True,
+        "z": "{cb35da1c-7ea4-44f0-8817-e3d80e8ba98c}",
         "z_active": True,
         "color_active": True,
-        "size": "MgO",
+        "size": "{41d51965-3670-43ba-8a10-d399070689e3}",
         "size_active": True,
         "refresh": True,
-        "refresh_trigger": True,
     }
 
     def __init__(self, **kwargs):
-        kwargs = self.apply_defaults(**kwargs)
-
+        self.defaults = self.update_defaults(**kwargs)
         self.scalings = {}
         self.lower_bounds = {}
         self.upper_bounds = {}
@@ -76,27 +91,17 @@ class Clustering(ScatterPlots):
             description="Analytics",
         )
         self.input_box = VBox([self.plotting_options])
-        # self.heatmap_fig = go.FigureWidget()
-        # self.heatmap_plot = interactive_output(
-        #     self.make_heatmap, {"channels": self.data, "show": self.plotting_options,}
-        # )
         self._refresh_clusters = Button(description="Refresh", button_style="warning")
         self.refresh_clusters.on_click(self.run_clustering)
         self.histogram_panel = VBox([self.channels_plot_options])
         self.boxplot_panel = VBox([self.channels_plot_options])
         self.stats_table = interactive_output(
             self.make_stats_table,
-            {"channels": self.data, "show": self.plotting_options,},
+            {
+                "channels": self.data,
+                "show": self.plotting_options,
+            },
         )
-
-        super().__init__(**kwargs)
-
-        # self.plotting_options = VBox(
-        #     [
-        #         self.plotting_options,
-        #
-        #     ]
-        # )
         self.ga_group_name.description = "Name"
         self.ga_group_name.value = "MyCluster"
         self.plotting_options.observe(self.show_trigger, names="value")
@@ -107,48 +112,27 @@ class Clustering(ScatterPlots):
 
         for ii in range(self.n_clusters.max):
             self.color_pickers[ii] = ColorPicker(
-                concise=False, description=("Color"), value=colors[ii],
+                concise=False,
+                description=("Color"),
+                value=colors[ii],
             )
             self.color_pickers[ii].uid = ii
             self.color_pickers[ii].observe(self.update_colormap, names="value")
             self.color_pickers[ii].observe(self.make_box_plot, names="value")
 
-        self.update_colormap(None, refresh_plot=False)
         self.custom_colormap = list(self.colormap.values())
-
         self.color.observe(self.check_color, names="value")
-
         self._clusters_options = Dropdown(
             description="Cluster", options=np.arange(self.n_clusters.max)
         )
         self.clusters_panel = VBox([self.clusters_options, self.color_pickers[0]])
         self.clusters_options.observe(self.clusters_panel_change, names="value")
         self.n_clusters.observe(self.run_clustering, names="value")
+        self.trigger.on_click(self.trigger_click)
 
-        self.update_choices(None)
-        self.run_clustering(None)
-        self.make_heatmap(None)
-        self.make_box_plot(None)
-        self.make_inertia_plot(None)
-        self.make_hist_plot(None)
+        super().__init__(**self.defaults)
 
-        self.trigger.on_click(self.save_cluster)
-        self._main = VBox(
-            [
-                self.project_panel,
-                VBox(
-                    [
-                        self.objects,
-                        self.data,
-                        self.downsampling,
-                        HBox([self.n_clusters, VBox([self.clusters_panel,])],),
-                    ]
-                ),
-                self.refresh_clusters,
-                self.input_box,
-                self.output_panel,
-            ]
-        )
+        self.refresh_clusters.click()
 
     @property
     def channels_plot_options(self):
@@ -160,6 +144,40 @@ class Clustering(ScatterPlots):
         """ipywidgets.Dropdown()"""
         return self._clusters_options
 
+    @property
+    def main(self):
+        """
+        :obj:`ipywidgets.VBox`: A box containing all widgets forming the application.
+        """
+        if self._main is None:
+            self._main = VBox(
+                [
+                    self.project_panel,
+                    VBox(
+                        [
+                            self.objects,
+                            self.data,
+                            self.downsampling,
+                            HBox(
+                                [
+                                    self.n_clusters,
+                                    VBox(
+                                        [
+                                            self.clusters_panel,
+                                        ]
+                                    ),
+                                ],
+                            ),
+                        ]
+                    ),
+                    self.refresh_clusters,
+                    self.input_box,
+                    self.output_panel,
+                ]
+            )
+
+        return self._main
+
     @property
     def mapping(self):
         """
@@ -195,40 +213,41 @@ class Clustering(ScatterPlots):
         """
         Update and display a specific plot.
         """
-        if self.plotting_options.value == "Statistics":
-            self.input_box.children = [
-                self.plotting_options,
-                self.stats_table,
-            ]
-        elif self.plotting_options.value == "Confusion Matrix":
-            self.make_heatmap(None)
-            self.input_box.children = [self.plotting_options, self.heatmap_fig]
-        elif self.plotting_options.value == "Crossplot":
-            self.input_box.children = [
-                self.plotting_options,
-                self.axes_options,
-                self.figure,
-            ]
-        elif self.plotting_options.value == "Histogram":
-            self.input_box.children = [
-                self.plotting_options,
-                self.histogram_panel,
-            ]
-            self.make_hist_plot(None)
-        elif self.plotting_options.value == "Boxplot":
-            self.make_box_plot(None)
-            self.input_box.children = [
-                self.plotting_options,
-                self.boxplot_panel,
-            ]
-        elif self.plotting_options.value == "Inertia":
-            self.make_inertia_plot(None)
-            self.input_box.children = [
-                self.plotting_options,
-                self.inertia_plot,
-            ]
+        try:
+            if self.plotting_options.value == "Statistics":
+                self.input_box.children = [
+                    self.plotting_options,
+                    self.stats_table,
+                ]
+            elif self.plotting_options.value == "Confusion Matrix":
+                self.make_heatmap(None)
+                self.input_box.children = [self.plotting_options, self.heatmap_fig]
+            elif self.plotting_options.value == "Crossplot":
+                self.input_box.children = [
+                    self.plotting_options,
+                    self.axes_options,
+                    self.figure,
+                ]
+            elif self.plotting_options.value == "Histogram":
+                self.input_box.children = [
+                    self.plotting_options,
+                    self.histogram_panel,
+                ]
+                self.make_hist_plot(None)
+            elif self.plotting_options.value == "Boxplot":
+                self.make_box_plot(None)
+                self.input_box.children = [
+                    self.plotting_options,
+                    self.boxplot_panel,
+                ]
+            elif self.plotting_options.value == "Inertia":
+                self.make_inertia_plot(None)
+                self.input_box.children = [
+                    self.plotting_options,
+                    self.inertia_plot,
+                ]
 
-        else:
+        except AttributeError:
             self.input_box.children = [
                 self.plotting_options,
             ]
@@ -248,7 +267,7 @@ class Clustering(ScatterPlots):
         """
         Change the colormap for clusters
         """
-        self.refresh_trigger.value = False
+        self.refresh.value = False
         self.colormap = {}
         for ii in range(self.n_clusters.value):
             colorpicker = self.color_pickers[ii]
@@ -267,13 +286,14 @@ class Clustering(ScatterPlots):
                 ]
 
         self.custom_colormap = list(self.colormap.values())
-        self.refresh_trigger.value = refresh_plot
+        self.refresh.value = refresh_plot
 
     def update_objects(self, _):
         """
         Reset all attributes on object change.
         """
         # Reset in all
+        self.refresh.value = False
         self.data_channels = {}
         self.clusters = {}
         self.scalings = {}
@@ -282,10 +302,18 @@ class Clustering(ScatterPlots):
         self.log_dict = {}
         self.histo_plots = {}
         self.box_plots = {}
-        self.channels_plot_options.options = []
+        self.channels_plot_options.options = None
         self.channels_plot_options.value = None
         self._mapping = None
+        self.figure.data = []
+        self.x_active.value = False
+        self.y_active.value = False
+        self.z_active.value = False
+        self.color_active.value = False
+        self.size_active.value = False
         self._indices = None
+        self.update_downsampling(None, refresh_plot=False)
+        self.refresh.value = True
 
         if self.n_values is not None:
             self.downsampling.max = self.n_values
@@ -299,7 +327,7 @@ class Clustering(ScatterPlots):
             return
 
         self.trigger.description = "Running ..."
-        self.refresh_trigger.value = False
+        self.refresh.value = False
 
         # Prime the app with clusters
         # Normalize values and run
@@ -311,7 +339,7 @@ class Clustering(ScatterPlots):
             vals[nns] = (
                 (vals[nns] - min(vals[nns]))
                 / (max(vals[nns]) - min(vals[nns]) + 1e-32)
-                * self.scalings[field].value
+                * self.scalings[dict(self.data.options)[field]].value
             )
             values += [vals]
 
@@ -332,7 +360,7 @@ class Clustering(ScatterPlots):
         self.trigger.description = "Export"
         self.refresh_clusters.description = "Refresh"
         self.show_trigger(None)
-        self.refresh_trigger.value = True
+        self.refresh.value = True
 
     def make_inertia_plot(self, _):
         """
@@ -347,15 +375,13 @@ class Clustering(ScatterPlots):
                 x=[self.n_clusters.value],
                 y=[self.clusters[self.n_clusters.value].inertia_],
             )
-            if self.static:
+
+            if getattr(self, "inertia_plot", None) is None:
                 self.inertia_plot = go.FigureWidget([line, point])
             else:
-                if getattr(self, "inertia_plot", None) is None:
-                    self.inertia_plot = go.FigureWidget([line, point])
-                else:
-                    self.inertia_plot.data = []
-                    self.inertia_plot.add_trace(line)
-                    self.inertia_plot.add_trace(point)
+                self.inertia_plot.data = []
+                self.inertia_plot.add_trace(line)
+                self.inertia_plot.add_trace(point)
             self.inertia_plot.update_layout(
                 {
                     "height": 300,
@@ -376,23 +402,24 @@ class Clustering(ScatterPlots):
             and getattr(self, "dataframe", None) is not None
         ):
             field = self.channels_plot_options.value
-            plot = go.Histogram(x=self.dataframe[field], histnorm="percent", name=field)
+            plot = go.Histogram(
+                x=self.dataframe[self.data.uid_name_map[field]],
+                histnorm="percent",
+                name=self.data.uid_name_map[field],
+            )
 
-            if self.static:
-                self.histo_plots[field] = go.FigureWidget([plot])
-            else:
-                if field not in self.histo_plots.keys():
-                    self.histo_plots[field] = go.FigureWidget()
+            if field not in self.histo_plots.keys():
+                self.histo_plots[field] = go.FigureWidget()
 
-                self.histo_plots[field].data = []
-                self.histo_plots[field].add_trace(plot)
-                self.histogram_panel.children = [
-                    self.channels_plot_options,
-                    self.scalings[field],
-                    HBox([self.lower_bounds[field], self.upper_bounds[field]]),
-                    #                 self.log_dict[field],
-                    self.histo_plots[field],
-                ]
+            self.histo_plots[field].data = []
+            self.histo_plots[field].add_trace(plot)
+            self.histogram_panel.children = [
+                self.channels_plot_options,
+                self.scalings[field],
+                HBox([self.lower_bounds[field], self.upper_bounds[field]]),
+                #                 self.log_dict[field],
+                self.histo_plots[field],
+            ]
 
     def make_box_plot(self, _):
         """
@@ -422,24 +449,21 @@ class Clustering(ScatterPlots):
                     )
                 )
 
-            if self.static:
-                self.box_plots[field] = go.FigureWidget(boxes)
-            else:
-                if field not in self.box_plots.keys():
-                    self.box_plots[field] = go.FigureWidget()
+            if field not in self.box_plots.keys():
+                self.box_plots[field] = go.FigureWidget()
 
-                self.box_plots[field].data = []
-                for box in boxes:
-                    self.box_plots[field].add_trace(box)
-                self.boxplot_panel.children = [
-                    self.channels_plot_options,
-                    self.box_plots[field],
-                ]
+            self.box_plots[field].data = []
+            for box in boxes:
+                self.box_plots[field].add_trace(box)
+            self.boxplot_panel.children = [
+                self.channels_plot_options,
+                self.box_plots[field],
+            ]
 
             self.box_plots[field].update_layout(
                 {
                     "xaxis": {"title": "Cluster #"},
-                    "yaxis": {"title": field},
+                    "yaxis": {"title": self.data.uid_name_map[field]},
                     "height": 600,
                     "width": 600,
                 }
@@ -471,14 +495,11 @@ class Clustering(ScatterPlots):
                 zsmooth=False,
             )
 
-            if self.static:
-                self.heatmap_fig = go.FigureWidget([plot])
-            else:
-                if getattr(self, "heatmap_fig", None) is None:
-                    self.heatmap_fig = go.FigureWidget()
+            if getattr(self, "heatmap_fig", None) is None:
+                self.heatmap_fig = go.FigureWidget()
 
-                self.heatmap_fig.data = []
-                self.heatmap_fig.add_trace(plot)
+            self.heatmap_fig.data = []
+            self.heatmap_fig.add_trace(plot)
 
             self.heatmap_fig.update_scenes(
                 aspectratio=dict(x=1, y=1, z=0.7), aspectmode="manual"
@@ -538,7 +559,7 @@ class Clustering(ScatterPlots):
                 yaxis={"autorange": "reversed"},
             )
 
-    def save_cluster(self, _):
+    def trigger_click(self, _):
         """
         Write cluster groups to the target geoh5 object.
         """
@@ -551,11 +572,7 @@ class Clustering(ScatterPlots):
             cluster_values[self._inactive_set] = 0
             for ii in range(self.n_clusters.value):
                 colorpicker = self.color_pickers[ii]
-
                 color = colorpicker.value.lstrip("#")
-
-                # group_map, color_map = {}, []
-                # for ind, group in self.time_groups.items():
                 group_map[ii + 1] = f"Cluster_{ii}"
                 color_map += [[ii + 1] + hex_to_rgb(color) + [1]]
 
@@ -606,7 +623,7 @@ class Clustering(ScatterPlots):
                 }
 
             if self.live_link.value:
-                self.live_link_output(obj)
+                self.live_link_output(self.export_directory.selected_path, obj)
 
             self.workspace.finalize()
 
@@ -622,7 +639,7 @@ class Clustering(ScatterPlots):
         if "kmeans" in self.data_channels.keys():
             del self.data_channels["kmeans"]
 
-        self.refresh_trigger.value = False
+        self.refresh.value = False
 
         for channel in self.data.value:
             self.get_channel(channel)
@@ -673,8 +690,6 @@ class Clustering(ScatterPlots):
                     (vals < self.lower_bounds[field].value)
                     | (vals > self.upper_bounds[field].value)
                 ] = np.nan
-
-                vals[(vals > 1e-38) * (vals < 2e-38)] = np.nan
                 values += [vals]
 
             values = np.vstack(values).T
@@ -691,10 +706,13 @@ class Clustering(ScatterPlots):
                 np.min([self.downsampling.value, len(active_set)]),
                 bandwidth=2.0,
                 rtol=1e0,
-                method="hist",
+                method="histogram",
             )
             self._indices = active_set[samples]
-            self.dataframe = pd.DataFrame(values[self.indices, :], columns=fields,)
+            self.dataframe = pd.DataFrame(
+                values[self.indices, :],
+                columns=[self.data.uid_name_map[field] for field in fields],
+            )
             tree = cKDTree(self.dataframe.values)
             inactive_set = np.ones(self.n_values, dtype="bool")
             inactive_set[self.indices] = False
@@ -711,7 +729,8 @@ class Clustering(ScatterPlots):
             self._mapping[inactive_set] = ind_out
             self._mapping[self.indices] = np.arange(self.indices.shape[0])
             self._inactive_set = np.where(np.all(np.isnan(values), axis=1))[0]
-            self.channels_plot_options.options = fields
+            options = [[self.data.uid_name_map[key], key] for key in fields]
+            self.channels_plot_options.options = options
 
         else:
             self.dataframe = None
@@ -721,3 +740,17 @@ class Clustering(ScatterPlots):
 
         self.update_axes(refresh_plot=refresh_plot)
         self.show_trigger(None)
+
+    def update_uid_name_map(self):
+        """
+        Update the dictionary that maps uuid to name.
+        """
+        uid_name = {}
+        for key, value in self.data.options:
+            if isinstance(value, UUID):
+                uid_name[value] = key
+            elif isinstance(value, str) and value in "XYZ":
+                uid_name[value] = value
+
+        uid_name["kmeans"] = "kmeans"
+        self.data.uid_name_map = uid_name
