@@ -1,64 +1,72 @@
-import re
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
 import os
-import osr
-from ipywidgets import SelectMultiple, VBox, HBox, Text, Textarea, Layout
+import re
+
+import matplotlib.pyplot as plt
 import numpy
+from fiona.transform import transform
 from geoh5py.data import FloatData
 from geoh5py.objects import Curve, Grid2D, Points, Surface
 from geoh5py.workspace import Workspace
-from geoapps.base import BaseApplication
-import matplotlib.pyplot as plt
-from fiona.transform import transform
-import gdal
-from geoapps.utils import (
-    export_grid_2_geotiff,
-    geotiff_2_grid,
-)
+from ipywidgets import HBox, Layout, SelectMultiple, Text, Textarea, VBox
+from osgeo import gdal, osr
+
 from geoapps.plotting import plot_plan_data_selection
+from geoapps.selection import ObjectDataSelection
+from geoapps.utils.utils import export_grid_2_geotiff, geotiff_2_grid
 
 
-class CoordinateTransformation(BaseApplication):
-    """
-
-    """
+class CoordinateTransformation(ObjectDataSelection):
+    """"""
 
     defaults = {
         "ga_group_name": "CoordinateTransformation",
         "h5file": "../../assets/FlinFlon.geoh5",
-        "objects": ["Gravity_Magnetics_drape60m", "Data_TEM_pseudo3D"],
+        "objects": [
+            "{538a7eb1-2218-4bec-98cc-0a759aa0ef4f}",
+            "{bb208abb-dc1f-4820-9ea9-b8883e5ff2c6}",
+        ],
         "code_in": "EPSG:26914",
         "code_out": "EPSG:4326",
     }
 
     def __init__(self, **kwargs):
 
-        kwargs = self.apply_defaults(**kwargs)
+        self.defaults = self.update_defaults(**kwargs)
 
         self.code_out.observe(self.set_wkt_out, names="value")
         self.code_in.observe(self.set_wkt_in, names="value")
         self.wkt_in.observe(self.set_authority_in, names="value")
         self.wkt_out.observe(self.set_authority_out, names="value")
 
-        super().__init__(**kwargs)
+        super().__init__(**self.defaults)
 
+        self.trigger.on_click(self.trigger_click)
         self.input_projection = HBox([self.code_in, self.wkt_in])
         self.output_projection = HBox([self.code_out, self.wkt_out])
-        self._main = VBox(
-            [
-                self.project_panel,
-                self.objects,
-                self.input_projection,
-                self.output_projection,
-                self.output_panel,
-            ]
-        )
 
-        def trigger_click(_):
-            self.trigger_click()
+    @property
+    def main(self):
+        if self._main is None:
+            self._main = VBox(
+                [
+                    self.project_panel,
+                    self.objects,
+                    self.input_projection,
+                    self.output_projection,
+                    self.output_panel,
+                ]
+            )
 
-        self.trigger.on_click(trigger_click)
+        return self._main
 
-    def trigger_click(self):
+    def trigger_click(self, _):
         """
         Run the coordinate transformation
         """
@@ -90,7 +98,9 @@ class CoordinateTransformation(BaseApplication):
                             )
                             grid = gdal.Open(temp_file)
                             gdal.Warp(
-                                temp_file_out, grid, dstSRS=self.wkt_out.value,
+                                temp_file_out,
+                                grid,
+                                dstSRS=self.wkt_out.value,
                             )
 
                             if count == 0:
@@ -102,12 +112,16 @@ class CoordinateTransformation(BaseApplication):
                                 )
                             else:
                                 _ = geotiff_2_grid(
-                                    temp_work, temp_file_out, grid_object=new_obj
+                                    temp_work, temp_file_out, grid=new_obj
                                 )
 
                             del grid
-                            os.remove(temp_file)
-                            os.remove(temp_file_out)
+                            if os.path.exists(temp_file):
+                                os.remove(temp_file)
+
+                            if os.path.exists(temp_file_out):
+                                os.remove(temp_file_out)
+
                             count += 1
 
                     new_obj.copy(parent=self.ga_group)
@@ -123,10 +137,12 @@ class CoordinateTransformation(BaseApplication):
                     if self.code_in.value == "EPSG:4326":
                         x, y = y, x
 
-                    x2, y2 = transform(self.wkt_in.value, self.wkt_out.value, x, y,)
-
-                    if self.code_out.value == "EPSG:4326":
-                        x2, y2 = y2, x2
+                    x2, y2 = transform(
+                        self.wkt_in.value,
+                        self.wkt_out.value,
+                        x,
+                        y,
+                    )
 
                     new_obj = obj.copy(parent=self.ga_group, copy_children=True)
                     new_obj.vertices = numpy.c_[x2, y2, obj.vertices[:, 2]]
@@ -144,7 +160,9 @@ class CoordinateTransformation(BaseApplication):
                         ax2.set_ylabel("Latitude")
 
             if self.live_link.value:
-                self.live_link_output(self.ga_group)
+                self.live_link_output(
+                    self.export_directory.selected_path, self.ga_group
+                )
             self.workspace.finalize()
 
     @property
@@ -206,15 +224,6 @@ class CoordinateTransformation(BaseApplication):
         self._h5file = workspace.h5file
         self.update_objects_list()
 
-    def update_objects_list(self):
-        if getattr(self, "_workspace", None) is not None:
-
-            self.objects.options = [""] + [
-                obj.name
-                for obj in self._workspace.all_objects()
-                if isinstance(obj, self.object_types)
-            ]
-
     def set_wkt_in(self, _):
         datasetSRS = osr.SpatialReference()
         datasetSRS.SetFromUserInput(self.code_in.value.upper())
