@@ -1,11 +1,23 @@
-import numpy as np
+#  Copyright (c) 2021 Mira Geoscience Ltd.
+#
+#  This file is part of geoapps.
+#
+#  geoapps is distributed under the terms and conditions of the MIT License
+#  (see LICENSE file at the root of this source code package).
+
+from __future__ import annotations
+
+from uuid import UUID
+
 import ipywidgets as widgets
-from ipywidgets import Dropdown, SelectMultiple, VBox, FloatText
-from geoh5py.workspace import Workspace
+import numpy as np
+from geoh5py.data import FloatData, IntegerData, ReferencedData
 from geoh5py.objects.object_base import ObjectBase
-from geoh5py.data import FloatData, IntegerData
+from geoh5py.workspace import Workspace
+from ipywidgets import Dropdown, FloatText, SelectMultiple, VBox
+
 from geoapps.base import BaseApplication
-from geoapps import utils
+from geoapps.utils import utils
 
 
 class ObjectDataSelection(BaseApplication):
@@ -14,19 +26,17 @@ class ObjectDataSelection(BaseApplication):
     """
 
     defaults = {}
+    _data = None
+    _objects = None
+    _add_groups = False
+    _select_multiple = False
+    _object_types = None
+    _find_label = []
 
     def __init__(self, **kwargs):
-        kwargs = self.apply_defaults(**kwargs)
-
-        self._add_groups = False
-        self._find_label = []
-        self._object_types = []
-        self._select_multiple = False
+        self._data_panel = None
 
         super().__init__(**kwargs)
-        self.data_panel = VBox([self.objects, self.data])
-        self.update_data_list(None)
-        self._main = self.data_panel
 
     @property
     def add_groups(self):
@@ -41,15 +51,22 @@ class ObjectDataSelection(BaseApplication):
         self._add_groups = value
 
     @property
-    def data(self):
+    def data(self) -> Dropdown | SelectMultiple:
         """
         Data selector
         """
         if getattr(self, "_data", None) is None:
             if self.select_multiple:
-                self._data = SelectMultiple(description="Data: ",)
+                self._data = SelectMultiple(
+                    description="Data: ",
+                )
             else:
-                self._data = Dropdown(description="Data: ",)
+                self._data = Dropdown(
+                    description="Data: ",
+                )
+            if self._objects is not None:
+                self.update_data_list(None)
+
         return self._data
 
     @data.setter
@@ -60,12 +77,31 @@ class ObjectDataSelection(BaseApplication):
         self._data = value
 
     @property
-    def objects(self):
+    def data_panel(self) -> VBox:
+        if getattr(self, "_data_panel", None) is None:
+            self._data_panel = VBox([self.objects, self.data])
+
+        return self._data_panel
+
+    @property
+    def main(self) -> VBox:
+        """
+        :obj:`ipywidgets.VBox`: A box containing all widgets forming the application.
+        """
+        self.__populate__(**self.defaults)
+        if self._main is None:
+            self._main = self.data_panel
+            self.update_data_list(None)
+
+        return self._main
+
+    @property
+    def objects(self) -> Dropdown:
         """
         Object selector
         """
         if getattr(self, "_objects", None) is None:
-            self.objects = Dropdown(description="Object:", options=[""])
+            self.objects = Dropdown(description="Object:")
 
         return self._objects
 
@@ -146,7 +182,7 @@ class ObjectDataSelection(BaseApplication):
             self._data = Dropdown(description="Data: ", options=options)
 
     @property
-    def workspace(self):
+    def workspace(self) -> Workspace | None:
         """
         Target geoh5py workspace
         """
@@ -184,14 +220,15 @@ class ObjectDataSelection(BaseApplication):
 
             data = []
             for value in values:
-                if obj.get_data(value):
-                    data += obj.get_data(value)
-
-                elif any([pg.name == value for pg in obj.property_groups]):
+                if any([pg.uid == value for pg in obj.property_groups]):
                     data += [
                         self.workspace.get_entity(prop)[0]
-                        for prop in obj.get_property_group(value).properties
+                        for prop in obj.find_or_create_property_group(
+                            name=self.data.uid_name_map[value]
+                        ).properties
                     ]
+                elif self.workspace.get_entity(value):
+                    data += self.workspace.get_entity(value)
 
             return obj, data
         else:
@@ -202,7 +239,6 @@ class ObjectDataSelection(BaseApplication):
         if getattr(self, "_workspace", None) is not None and self._workspace.get_entity(
             self.objects.value
         ):
-
             for entity in self._workspace.get_entity(self.objects.value):
                 if isinstance(entity, ObjectBase):
                     obj = entity
@@ -210,39 +246,37 @@ class ObjectDataSelection(BaseApplication):
             if getattr(obj, "get_data_list", None) is None:
                 return
 
-            options = [""]
+            options = [["", None]]
 
             if (self.add_groups or self.add_groups == "only") and obj.property_groups:
                 options = (
                     options
-                    + ["-- Groups --"]
-                    + [p_g.name for p_g in obj.property_groups]
+                    + [["-- Groups --", None]]
+                    + [[p_g.name, p_g.uid] for p_g in obj.property_groups]
                 )
 
             if self.add_groups != "only":
-                data_list = obj.get_data_list()
-                options = (
-                    options
-                    + ["--- Channels ---"]
-                    + [
-                        obj.get_data(uid)[0].name
-                        for uid in data_list
-                        if isinstance(obj.get_data(uid)[0], (IntegerData, FloatData))
-                    ]
-                    + ["Z"]
-                )
+                options += [["--- Channels ---", None]]
+                for child in obj.children:
+                    if isinstance(child, (IntegerData, FloatData)):
+                        options += [[child.name, child.uid]]
+
+                options += [["X", "X"], ["Y", "Y"], ["Z", "Z"]]
 
             value = self.data.value
             self.data.options = options
 
+            self.update_uid_name_map()
+
             if self.select_multiple and any([val in options for val in value]):
                 self.data.value = [val for val in value if val in options]
-            elif value in options:
+            elif value in dict(options).values():
                 self.data.value = value
             elif self.find_label:
                 self.data.value = utils.find_value(self.data.options, self.find_label)
         else:
             self.data.options = []
+            self.data.uid_name_map = {}
 
         self.refresh.value = True
 
@@ -251,15 +285,19 @@ class ObjectDataSelection(BaseApplication):
             value = self.objects.value
 
             if len(self.object_types) > 0:
-                options = [""] + [
-                    obj.name
-                    for obj in self._workspace.all_objects()
+                obj_list = [
+                    obj
+                    for obj in self._workspace.objects
                     if isinstance(obj, self.object_types)
                 ]
             else:
-                options = [""] + list(self._workspace.list_objects_name.values())
+                obj_list = self._workspace.objects
 
-            if value in options:  # Silent update
+            options = [["", None]] + [
+                [obj.parent.name + "/" + obj.name, obj.uid] for obj in obj_list
+            ]
+
+            if value in list(dict(options).values()):  # Silent update
                 self.objects.unobserve(self.update_data_list, names="value")
                 self.objects.options = options
                 self.objects.value = value
@@ -267,6 +305,18 @@ class ObjectDataSelection(BaseApplication):
             else:
                 self.objects.options = options
 
+    def update_uid_name_map(self):
+        """
+        Update the dictionary that maps uuid to name.
+        """
+        uid_name = {}
+        for key, value in self.data.options:
+            if isinstance(value, UUID):
+                uid_name[value] = key
+            elif isinstance(value, str) and value in "XYZ":
+                uid_name[value] = value
+        self.data.uid_name_map = uid_name
+
 
 class LineOptions(ObjectDataSelection):
     """
@@ -274,23 +324,24 @@ class LineOptions(ObjectDataSelection):
     """
 
     defaults = {"find_label": "line"}
+    _multiple_lines = None
 
     def __init__(self, **kwargs):
-        kwargs = self.apply_defaults(**kwargs)
 
-        self._multiple_lines = None
+        self.defaults = self.update_defaults(**kwargs)
 
-        super().__init__(**kwargs)
+        super().__init__(**self.defaults)
 
-        if "objects" in kwargs.keys() and isinstance(kwargs["objects"], Dropdown):
-            self._objects.observe(self.update_data_list, names="value")
+        self.objects.observe(self.update_data_list, names="value")
+        self.data.observe(self.update_line_list, names="value")
+        self.data.description = "Lines field"
 
-        self._data.observe(self.update_line_list, names="value")
-        self.update_data_list(None)
-        self.update_line_list(None)
+    @property
+    def main(self):
+        if self._main is None:
+            self._main = VBox([self._data, self.lines])
 
-        self._main = VBox([self._data, self.lines])
-        self._data.description = "Lines field"
+        return self._main
 
     @property
     def lines(self):
@@ -299,9 +350,13 @@ class LineOptions(ObjectDataSelection):
         """
         if getattr(self, "_lines", None) is None:
             if self.multiple_lines:
-                self._lines = widgets.SelectMultiple(description="Select lines:",)
+                self._lines = widgets.SelectMultiple(
+                    description="Select lines:",
+                )
             else:
-                self._lines = widgets.Dropdown(description="Select line:",)
+                self._lines = widgets.Dropdown(
+                    description="Select line:",
+                )
 
         return self._lines
 
@@ -322,7 +377,10 @@ class LineOptions(ObjectDataSelection):
     def update_line_list(self, _):
         _, data = self.get_selected_entities()
         if data and getattr(data[0], "values", None) is not None:
-            self.lines.options = [""] + np.unique(data[0].values).tolist()
+            if isinstance(data[0], ReferencedData):
+                self.lines.options = [""] + list(data[0].value_map.map.values())
+            else:
+                self.lines.options = [""] + np.unique(data[0].values).tolist()
 
 
 class TopographyOptions(ObjectDataSelection):
@@ -330,30 +388,28 @@ class TopographyOptions(ObjectDataSelection):
     Define the topography used by the inversion
     """
 
-    def __init__(self, **kwargs):
+    def __init__(
+        self, option_list=["None", "Object", "Relative to Sensor", "Constant"], **kwargs
+    ):
+        self.defaults = self.update_defaults(**kwargs)
         self.find_label = ["topo", "dem", "dtm", "elevation", "Z"]
         self._offset = FloatText(description="Vertical offset (+ve up)")
-        self._constant = FloatText(description="Elevation (m)",)
-
-        super().__init__(**kwargs)
-
-        self.objects.value = utils.find_value(self.objects.options, self.find_label)
+        self._constant = FloatText(
+            description="Elevation (m)",
+        )
         self.option_list = {
-            "Object": self.main,
-            "Relative to Sensor": self.offset,
-            "Constant": self.constant,
             "None": widgets.Label("No topography"),
+            "Object": self.data_panel,
+            "Relative to Sensor": self._offset,
+            "Constant": self._constant,
         }
         self._options = widgets.RadioButtons(
-            options=["Object", "Relative to Sensor", "Constant"],
+            options=option_list,
             description="Define by:",
         )
+        self.options.observe(self.update_options)
 
-        def update_options(_):
-            self.update_options()
-
-        self.options.observe(update_options)
-        self._main = VBox([self.options, self.option_list[self.options.value]])
+        super().__init__(**self.defaults)
 
     @property
     def panel(self):
@@ -363,6 +419,13 @@ class TopographyOptions(ObjectDataSelection):
     def constant(self):
         return self._constant
 
+    @property
+    def main(self):
+        if self._main is None:
+            self._main = VBox([self.options, self.option_list[self.options.value]])
+
+        return self._main
+
     @property
     def offset(self):
         return self._offset
@@ -371,8 +434,8 @@ class TopographyOptions(ObjectDataSelection):
     def options(self):
         return self._options
 
-    def update_options(self):
-        self._main.children = [
+    def update_options(self, _):
+        self.main.children = [
             self.options,
             self.option_list[self.options.value],
         ]
