@@ -1,4 +1,4 @@
-from SimPEG import Maps, Survey, Utils
+from geoapps.simpegPF import Maps, Survey, Utils
 import numpy as np
 import scipy.sparse as sp
 from scipy.constants import mu_0
