@@ -212,6 +212,7 @@ class BetaEstimate_ByEig(InversionDirective):
         t = np.dot(x0, self.dmisfit.deriv2(m, x0, f=f))
         b = np.dot(x0, self.reg.deriv2(m, v=x0))
 
+
         self.beta0 = self.beta0_ratio * (t / b)
 
         self.invProb.beta = self.beta0
@@ -794,6 +795,8 @@ class SaveIterationsGeoH5(InversionDirective):
     data_type = {}
     replace_values = False
     no_data_value = None
+    group = False
+    reg_fun = ["phi_ms", "phi_msx", "phi_msy", "phi_msz"]
 
     def initialize(self):
 
@@ -815,6 +818,9 @@ class SaveIterationsGeoH5(InversionDirective):
 
         prop = self.check_mvi_format(prop)
 
+        if self.group:
+            group_name = "Iteration_0"
+
         for ii, channel in enumerate(self.channels):
 
             attr = prop[ii :: len(self.channels)]
@@ -822,31 +828,43 @@ class SaveIterationsGeoH5(InversionDirective):
             if self.sorting is not None:
                 attr = attr[self.sorting]
 
-            data = self.h5_object.add_data(
-                {
-                    f"Iteration_0_"
-                    + channel: {"association": self.association, "values": attr}
-                }
-            )
+            if channel not in list(self.data_type.keys()):
+                data = self.h5_object.add_data(
+                    {
+                        f"Iteration_0_"
+                        + channel: {"association": self.association, "values": attr}
+                    }
+                )
+                data.entity_type.name = channel
+                self.data_type[channel] = data.entity_type
+            else:
+                data = self.h5_object.add_data(
+                    {
+                        f"Iteration_0_"
+                        + channel: {
+                            "association": self.association,
+                            "values": attr,
+                            "entity_type": self.data_type[channel],
+                        }
+                    }
+                )
 
-            data.entity_type.name = channel
-            self.data_type[channel] = data.entity_type
+            if self.group:
+                self.h5_object.add_data_to_group(data, group_name)
 
         if self.save_objective_function:
-            regCombo = ["phi_ms", "phi_msx"]
-
-            if self.prob[0].mesh.dim >= 2:
-                regCombo += ["phi_msy"]
-
-            if self.prob[0].mesh.dim == 3:
-                regCombo += ["phi_msz"]
-
             # Save the data.
             iterDict = {"beta": f"{self.invProb.beta:.3e}"}
             iterDict["phi_d"] = f"{self.invProb.phi_d:.3e}"
-            iterDict["phi_m"] = f"{self.invProb.phi_m:.3e}"
 
-            for label, fcts in zip(regCombo, self.reg.objfcts[0].objfcts):
+            if isinstance(self.invProb.phi_m, (list, np.ndarray)):
+                phi_m = np.sum(self.invProb.phi_m)
+            else:
+                phi_m = self.invProb.phi_m
+
+            iterDict["phi_m"] = f"{phi_m:.3e}"
+
+            for label, fcts in zip(self.reg_fun, self.reg.objfcts[0].objfcts):
                 iterDict[label] = f"{fcts(self.invProb.model):.3e}"
 
             self.h5_object.parent.add_comment(
@@ -874,6 +892,9 @@ class SaveIterationsGeoH5(InversionDirective):
 
         prop = self.check_mvi_format(prop)
 
+        if self.group:
+            group_name = f"Iteration_{self.opt.iter}"
+
         for ii, channel in enumerate(self.channels):
             attr = prop[ii :: len(self.channels)]
 
@@ -889,7 +910,7 @@ class SaveIterationsGeoH5(InversionDirective):
                 data.name = f"Iteration_{self.opt.iter}_" + channel
                 data.values = attr
             else:
-                self.h5_object.add_data(
+                data = self.h5_object.add_data(
                     {
                         f"Iteration_{self.opt.iter}_"
                         + channel: {
@@ -900,15 +921,10 @@ class SaveIterationsGeoH5(InversionDirective):
                     }
                 )
 
+            if self.group:
+                self.h5_object.add_data_to_group(data, group_name)
+
         if self.save_objective_function:
-            regCombo = ["phi_ms", "phi_msx"]
-
-            if self.prob[0].mesh.dim >= 2:
-                regCombo += ["phi_msy"]
-
-            if self.prob[0].mesh.dim == 3:
-                regCombo += ["phi_msz"]
-
             # Save objective function.
             if isinstance(self.invProb.beta, float):
                 beta = self.invProb.beta
@@ -922,15 +938,15 @@ class SaveIterationsGeoH5(InversionDirective):
             else:
                 phi_d = self.invProb.phi_d[0]
 
-            if isinstance(self.invProb.phi_m, float):
-                phi_m = self.invProb.phi_m
+            if isinstance(self.invProb.phi_m, (list, np.ndarray)):
+                phi_m = np.sum(self.invProb.phi_m)
             else:
-                phi_m = self.invProb.phi_m[0]
+                phi_m = self.invProb.phi_m
 
             iterDict["phi_d"] = f"{phi_d:.3e}"
             iterDict["phi_m"] = f"{phi_m:.3e}"
 
-            for label, fcts in zip(regCombo, self.reg.objfcts[0].objfcts):
+            for label, fcts in zip(self.reg_fun, self.reg.objfcts[0].objfcts):
                 iterDict[label] = f"{fcts(self.invProb.model):.3e}"
 
             self.h5_object.parent.add_comment(
@@ -1209,7 +1225,7 @@ class Update_IRLS(InversionDirective):
                 self.opt.iter -= 1
                 return
 
-        elif np.all([self.mode == 1, self.opt.iter % self.coolingRate == 0]):
+        elif np.all([self.mode == 1, (self.opt.iter % self.coolingRate) == 0]):
 
             self.invProb.beta = self.invProb.beta / self.coolingFactor
 
