@@ -5,8 +5,8 @@ from scipy.interpolate import griddata
 from scipy.spatial import cKDTree
 from scipy.interpolate.interpnd import _ndim_coords_from_arrays
 from matplotlib.colors import LightSource, Normalize
-import matplotlib.gridspec as gridspec
-from SimPEG.Utils import mkvc
+
+from .matutils import mkvc
 
 
 def plot2Ddata(
